/* RCS: $Id: baryFase.c,v 3.4 2004/10/06 16:31:25 arots Exp arots $ */
/*-----------------------------------------------------------------------
 *
 *  baryFase.c
 *
 *  Date: 21 November 1997
 *
 *  Arnold Rots, USRA
 *
 *  baryFase is a utility program that interactively calculates
 *  barycenter corrections and absolute phases for individual times.
 *
 *  Usage:
 *    baryFase [-d[ebug]]
 *
 *----------------------------------------------------------------------*/

#include "bary.h"

char *bF = "baryFase Version 1.0.0" ;
int main (int argc, char *argv[])
{
  int debug = 0 ;
  char line[80] ;
  char oefile[1024] ;
  int denum = 0 ;
  char source[32] ;
  int nosource = 0 ;
  int nophase = 0 ;
  PsrTimPar ptp ;
  PsrBinPar pbp ;
  double t, dt, f ;
  int mjd ;
  long mrefi = 0 ;
  double mreff =0.0 ;
  char tu[12] ;
  double roffset = 0.0 ;
  int bin = 0 ;
  MJDTime mjdtime ;
  double dir[3] ;
  int error=0 ;
  char fromTS[12] ;
  int bc = 1 ;

  if ( ( argc > 1 ) && ( !strncmp (argv[1], "-d", 2) ) )
    debug = 1 ;

  /*
   *       -----------------------
   *    -- Get mission/observatory --
   *       -----------------------
   *
   *       (only observatories defined in bary.h and
   *        supported in bary.c may be specified)
   */
  printf ("Mission/observatory [Geocenter]: ") ;
  gets (line) ;
  if ( !(*line) )
    mission = Geocenter ;
  else if ( !strncmp (line, "xte", 3) || !strncmp (line, "XTE", 3) )
    mission = XTE ;
  else if ( !strncmp (line, "cxo", 3) || !strncmp (line, "CXO", 3) ||
	    !strncmp (line, "axaf", 4) || !strncmp (line, "AXAF", 4) ||
	    !strncmp (line, "chandra", 7) || !strncmp (line, "CHANDRA", 7) )
    mission = AXAF ;
  else if ( !strncmp (line, "geo", 3) || !strncmp (line, "GEO", 3) ||
	    !strncmp (line, "Geo", 3) )
    mission = Geocenter ;
  else if ( !strncmp (line, "bar", 3) || !strncmp (line, "BAR", 3) ||
	    !strncmp (line, "Bar", 3) ) {
    mission = Geocenter ;
    bc = 0 ;
  }
  else {
    fprintf (stderr, "%s is an unknown mission\n", line) ;
    exit (1) ;
  }

  /*
   *       ------------------------
   *    -- Get orbit ephemeris file --
   *       ------------------------
   */
  if ( mission != Geocenter ) {
    printf ("Orbit ephemeris file [none]: ") ;
    gets (oefile) ;
    if ( !(*oefile) || ( strncmp (oefile, "None", 4) ) ||
	 ( strncmp (oefile, "none", 4) ) || ( strncmp (oefile, "NONE", 4) ) )
      mission = Geocenter ;
  }

  /*
   *       ------------------------
   *    -- Get JPL ephemeris number --
   *       ------------------------
   *
   *       (200 or 405; <cr> or 0 uses the local default)
   */
  printf ("JPL ephemeris [default]: ") ;
  gets (line) ;
  if ( *line )
    denum = atoi (line) ;
  else
    denum = 0 ;

  /*
   *       --------------------
   *    -- Get input time scale --
   *       --------------------
   *
   *       (TT, TAI, UTC, TDB)
   */
  printf ("Input Time System [TT]: ") ;
  gets (line) ;
  if ( *line )
    strncpy (fromTS, line, 4) ;
  else
    strcpy (fromTS, "TT") ;
  if ( !strncmp(fromTS, "TDB", 3) )
    bc = 0 ;

  /*
   *       -------------------
   *    -- Get input time unit --
   *       -------------------
   *
   *       (time may be specified as MET in seconds or days,
   *        or in MJD)
   */
  printf ("Input in MET secs, MET days, or MJD (s/d/m) [m]: ") ;
  gets (line) ;
  if ( ( *line == 's' ) || (*line == 'S' ) ) {
    mjd = 0 ;
    strcpy (tu, "s") ;
  }
  else if ( ( *line == 'd' ) || (*line == 'D' ) ) {
    mjd = 0 ;
    strcpy (tu, "d") ;
  }
  else {
    mjd = 1 ;
    strcpy (tu, "d") ;
    mrefi = 0 ;
    mreff = 0.0 ;
  }

  /*
   *       -----------------
   *    -- Get reference MJD --
   *       -----------------
   *
   *       (for MET we need a reference MET; default: XTE;
   *        the default may depend on mission)
   */
  if ( !mjd ) {
    printf ("Reference MJD: integer and fractional part: ") ;
    gets (line) ;
    if ( *line )
      sscanf (line, "%d %lg", &mrefi, &mreff) ;
    else {
      mrefi = 49353 ;
      mreff = 60.184 / 86400.0 ;
    }
  }

  /*
   *       ---------------
   *    -- Get source name --
   *       ---------------
   *
   *       (source name under which to find timing ephemeris
   *        in psrtime.dat)
   */
  printf ("Source [none]: ") ;
  gets (source) ;
  if ( !(*source) || ( !strncmp (source, "None", 4) ) ||
       ( !strncmp (source, "none", 4) ) || ( !strncmp (source, "NONE", 4) ) )
    nosource = 1 ;

  /*
   *       ------------------------
   *    -- Binary orbit correction? --
   *       ------------------------
   */
  printf ("Binary orbit correction (y/n) [n]: ") ;
  gets (line) ;
  if ( ( *line == 'y' ) || ( *line =='Y' ) )
    bin = 1 ;
  else
    bin = 0 ;

  /*
   *       ---------------------
   *    -- Get timing parameters --
   *       ---------------------
   *
   *       (if the pulsar database is not to be used, provide
   *        the timing parameters here; if only RA and Dec are
   *        provided, we will just do barycenter corrections)
   */
  if ( nosource ) {
    printf ("Timing ephemeris: RA, Dec, (t(0,geo), nu, nu-dot, nu-double-dot, nu-triple-dot:\n") ;
    gets (line) ;
    if ( *line ) {
      if ( ( sscanf (line, "%lg %lg %d.%lg %lg %lg %lg %lg", &(ptp.ra), &(ptp.dec),
	      &(ptp.t0.MJDint), &(ptp.t0.MJDfr),
	      &(ptp.f), &(ptp.df), &(ptp.ddf), &(ptp.dddf)) ) < 3 )
	nophase = 1 ;
      else
	nophase = 0 ;
      dir[0] = cos(ptp.ra/RADEG) * cos(ptp.dec/RADEG) ;
      dir[1] = sin(ptp.ra/RADEG) * cos(ptp.dec/RADEG) ;
      dir[2] = sin(ptp.dec/RADEG) ;
      if ( ( denum == 405 ) && ( ptp.denum == 200 ) )
	c200to405 (dir, debug) ;
    }
    else
      nophase = 1 ;
  }
  else
    nophase = 0 ;

  /*
   *       -----------------
   *    -- Initialize things --
   *       -----------------
   */
  error = baryinit (mission, fromTS, "TDB", mrefi, mreff, tu, denum) ;
  if ( denum && ( error != denum ) )
    printf ("Wrong ephemeris (%d)\n", error) ;
  else {
    denum = error ;
    printf ("Using JPL DE%d\n", denum) ;
    error = 0 ;
  }

  /*
   *       -------------
   *    -- The time loop --
   *       -------------
   */
  while ( !error ) {
    printf ("Type MET or MJD: ") ;
    gets (line) ;
    if ( !(*line) )
      break ;
    if ( !sscanf (line, "%lg", &t) || ( t < -1.0 ) )
      break ;
    met2mjd (t, &mjdtime) ;
    printf ("MJD: %d + %f\n",
	    mjdtime.MJDint, mjdtime.MJDfr) ;

    if ( !nophase ) {
      if ( !nosource ) {
	error = timeparms (source, &mjdtime, roffset, bin, &ptp, &pbp, debug) ;
	dir[0] = cos(ptp.ra/RADEG) * cos(ptp.dec/RADEG) ;
	dir[1] = sin(ptp.ra/RADEG) * cos(ptp.dec/RADEG) ;
	dir[2] = sin(ptp.dec/RADEG) ;
	if ( ( denum == 405 ) && ( ptp.denum == 200 ) )
	  c200to405 (dir, debug) ;
      }
    }
    if ( bc )
      dt = barycorr (&mjdtime, dir, scorbit(oefile, &mjdtime, &error)) ;
    else
      dt = 0.0 ;
    printf ("Time = %19.17g: Barycorr  = %f s\n", t, dt) ;
    if ( !nophase ) {
      if ( !error ) {
	dt += SECDAY * ((mjdtime.MJDint - ptp.t0.MJDint)
			+ mjdtime.MJDfr - ptp.t0.MJDfr) ;
	f = ptp.f + dt * ptp.df + 0.5 * dt * dt * ptp.ddf + dt * dt * dt * ptp.dddf / 6.0 ;
	printf ("Time = %19.17g: T - epoch = %f s\n", t, dt) ;
	printf ("Time = %19.17g: Frequency = %18.17g Hz\n", t, f) ;
	printf ("Time = %19.17g: Period    = %18.17g s\n", t, 1.0/f) ;
	dt = absphase (&mjdtime, &ptp, bin, &pbp, bc, dir, oefile, &error) ;
	printf ("Time = %19.17g: AbsPhase  = %f\n", t, dt) ;
      }
    }
  }
  exit (error) ;
}
