/* RCS: $Id: fBadd.c,v 3.4 2004/10/06 16:30:32 arots Exp arots $ */
/*-----------------------------------------------------------------------
 *
 *  fBadd.c
 *
 *  Date: 20 March 1997
 *
 *  Arnold Rots, USRA
 *
 *  fBadd allows adding several fB files together.  Errors are added in
 *  quadrature.
 *
 *  6 August 2003: proper handling of offsets
 *
 *----------------------------------------------------------------------*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fitsio.h>
#include <longnam.h>
#define NCHAN   4096

char *fB = "fBadd $Revision: 3.4 $" ;
int main (int argc, char *argv[])
{

  fitsfile *fits_infile;
  fitsfile *fits_outfile;
  char line[1024] ;
  char progName[256] ;
  char *dataFile[1024] ;
  char outFile[1025] ;
  char keystrval[80] ;
  char comment[80] ;
  int nfiles = 0 ;
  int error = 0 ;
  int htype ;
  int offset[100] ;
  long nrows, nch, i, j, jj, n1, n2 ;
  long nrows0, nch0, k, m ;
  double nulv = 0.0 ;
  int any ;
  double dnphase, expos ;
  double dnphase0, expos0 ;
  double phase ;
  double tstart, tstop, t1, t2 ;
  double inspec[NCHAN] ;
  double outspec[NCHAN] ;

  int getopts (int, char **, char *, char **, int *, char *, int *) ;

/*
 *  Decode command line options -----------------------------------------
 */
  printf ("# ") ;
  for (i=0; i<argc; i++)
    printf (" %s", argv[i]) ;
  printf ("\n") ;
  if ( getopts (argc, argv, progName, dataFile, &nfiles, outFile, offset) )
    exit (1) ;

/*
 *  Copy the input file -------------------------------------------------
 */
  if ( !strstr (outFile, ".") )
    strcat (outFile, ".pha") ;
  sprintf (line, "cp %s %s", dataFile[0], outFile) ;
  if ( system (line) ) {
    fprintf (stderr,"%s: Could not open input file %s or create output file %s\n",
	     progName, dataFile[0], outFile) ;
    exit (1) ;
  }

/*
 *  Fix up output file --------------------------------------------------
 */
  if ( fits_open_file (&fits_outfile, outFile, READWRITE, &error) ) {
    fprintf (stderr,"%s: Failed to re-open output file %s\n",
	     progName, outFile) ;
    exit (2) ;
  }
  fits_write_date (fits_outfile, &error) ;
  fits_write_chksum (fits_outfile, &error) ;

/*
 *  Check first extension -----------------------------------------------
 */
  fits_movabs_hdu (fits_outfile, 2, &htype, &error) ;
  fits_read_key (fits_outfile, TSTRING, "HDUCLAS5", keystrval, comment, &error) ;
  if ( strcmp (keystrval, "fB") ) {
    fprintf (stderr,"%s: Input file %s is of the wrong type\n",
	     progName, dataFile[0]) ;
    fits_delete_file (fits_outfile, &error) ;
    exit (3) ;
  }
  fits_read_key_lng (fits_outfile,        "NAXIS2",    &nrows0, comment, &error) ;
  fits_read_key_dbl (fits_outfile,      "PHASEDEL",  &dnphase0, comment, &error) ;
  fits_read_key_dbl (fits_outfile,        "TSTART",    &tstart, comment, &error) ;
  fits_read_key_dbl (fits_outfile,         "TSTOP",     &tstop, comment, &error) ;
  fits_read_key_lng (fits_outfile,      "DETCHANS",      &nch0, comment, &error) ;

/*
 *  Input file loop -----------------------------------------------------
 */
  for (k=1; k<nfiles; k++) {
    if ( fits_open_file (&fits_infile, dataFile[k], READONLY, &error) ) {
      fprintf (stderr,"%s: Failed to open input file %s\n",
	       progName, dataFile[k]) ;
      continue ;
    }
    m = offset[k] ;

/*
 *  Check first extension -----------------------------------------------
 */
    fits_movabs_hdu (fits_infile, 2, &htype, &error) ;
    fits_read_key (fits_infile, TSTRING, "HDUCLAS5", keystrval, comment, &error) ;
    if ( strcmp (keystrval, "fB") ) {
      fprintf (stderr,"%s: Input file %s is of the wrong type\n",
	       progName, dataFile[k]) ;
      continue ;
    }
    fits_read_key_lng (fits_infile,        "NAXIS2",    &nrows, comment, &error) ;
    fits_read_key_dbl (fits_infile,      "PHASEDEL",  &dnphase, comment, &error) ;
    fits_read_key_dbl (fits_outfile,       "TSTART",       &t1, comment, &error) ;
    fits_read_key_dbl (fits_outfile,        "TSTOP",       &t2, comment, &error) ;
    fits_read_key_lng (fits_infile,      "DETCHANS",      &nch, comment, &error) ;

    if ( ( nrows != nrows0 ) || ( nch != nch0 ) || ( dnphase != dnphase0 ) ) {
      fprintf (stderr,"%s: Input file %s is immensurate with previous file(s)\n",
	       progName, dataFile[k]) ;
      continue ;
    }
    if ( t1 < tstart )
      tstart = t1 ;
    if (t2 > tstop )
      tstop = t2 ;
  
/*
 *  Enter new values -----------------------------------------------------
 */
    for (j=1; j<=nrows; j++) {
      if ( m ) {
	jj = j + m ;
	if ( jj <= 0 )
	  jj += nrows ;
	else if ( jj > nrows )
	  jj -= nrows ;
      }
      else
	jj = j ;
/*
 *  Counts -----------------------------------------------------
 */
      fits_read_col ( fits_infile, TDOUBLE, 4, jj, 1, nch, &nulv,  inspec,
		     &any, &error) ;
      fits_read_col (fits_outfile, TDOUBLE, 4, j, 1, nch, &nulv, outspec,
		     &any, &error) ;
      for (i=0; i<nch; i++)
	outspec[i] += inspec[i] ;
      fits_write_col_dbl (fits_outfile, 4, j, 1, nch, outspec, &error) ;
/*
 *  Errors -----------------------------------------------------
 */
      fits_read_col ( fits_infile, TDOUBLE, 5, jj, 1, nch, &nulv,  inspec,
		     &any, &error) ;
      fits_read_col (fits_outfile, TDOUBLE, 5, j, 1, nch, &nulv, outspec,
		     &any, &error) ;
      for (i=0; i<nch; i++)
	outspec[i] = sqrt (outspec[i]*outspec[i] + inspec[i]*inspec[i]) ;
      fits_write_col_dbl (fits_outfile, 5, j, 1, nch, outspec, &error) ;
/*
 *  Baseline ---------------------------------------------------
 */
      fits_read_col ( fits_infile, TDOUBLE, 6, jj, 1, nch, &nulv,  inspec,
		     &any, &error) ;
      fits_read_col (fits_outfile, TDOUBLE, 6, j, 1, nch, &nulv, outspec,
		     &any, &error) ;
      for (i=0; i<nch; i++)
	outspec[i] += inspec[i] ;
      fits_write_col_dbl (fits_outfile, 6, j, 1, nch, outspec, &error) ;
/*
 *  Exposure ---------------------------------------------------
 */
      fits_read_col ( fits_infile, TDOUBLE, 7, jj, 1,   1, &nulv,  &expos,
		     &any, &error) ;
      fits_read_col (fits_outfile, TDOUBLE, 7, j, 1,   1, &nulv, &expos0,
		     &any, &error) ;
      expos0 += expos ;
      fits_write_col_dbl (fits_outfile, 7, j, 1,   1, &expos0, &error) ;
    }
    fits_close_file (fits_infile, &error) ;
  }  
/*
 *  Last fixes and close ------------------------------------------------
 */
  fits_write_date (fits_outfile, &error) ;
  fits_modify_key_str (fits_outfile, "CREATOR",    fB,     "&", &error) ;
  fits_modify_key_dbl (fits_outfile, "TSTART", tstart, 12, "&", &error) ;
  fits_modify_key_dbl (fits_outfile,  "TSTOP",  tstop, 12, "&", &error) ;
  fits_write_chksum (fits_outfile, &error) ;

  fits_close_file (fits_outfile, &error) ;

  exit (error) ;
}

/*-----------------------------------------------------------------------
 *
 *  getopts decodes command line options for faseBin.
 *
 *----------------------------------------------------------------------*/

int getopts (int argc, char **argv, char *progName, char **dataFile,
	     int *nfiles, char *outFile, int *offset)
{
  int i ;
  FILE *F ;
  char name[1025] ;
/*
 *  Initialize ----------------------------------------------------------
 */
  outFile[0] = 0 ;
  strcpy (progName, *argv) ;
  argv++ ;
  argc-- ;
  for (i=0; i<100; i++)
    offset[i] = 0 ;

/*
 *  Decode --------------------------------------------------------------
 */
  while ( argc > 0 ) {
    if ( !strcmp (*argv, "-i") ) {
      argc-- ;
      while ( argc && ( **(++argv) != '-' ) ) {
	dataFile[*nfiles] = (char*) malloc(1025 * sizeof(char)) ;
	strcpy (dataFile[*nfiles], *argv) ;
	argc-- ;
	(*nfiles)++ ;
      }
      argv-- ;
      argc++ ;
    }
    else if ( !strcmp (*argv, "-d") ) {
      argc-- ;
      i=0 ;
      while ( argc && ( **(++argv) != '-' ) ) {
	offset[i] = atoi (*argv) ;
	argc-- ;
	i++ ;
      }
      argv-- ;
      argc++ ;
    }
    if ( !strcmp (*argv, "-f") ) {
      argc-- ;
      F = fopen (*(++argv), "r") ;
      while ( fscanf (F, "%s %d", name, offset+(*nfiles)) == 2 ) {
	dataFile[*nfiles] = (char*) malloc(1025 * sizeof(char)) ;
	strcpy (dataFile[*nfiles], name) ;
	(*nfiles)++ ;
      }
      fclose (F) ;
    }
    else if ( !strcmp (*argv, "-o") ) {
      strcpy (outFile, *(++argv)) ;
      argc-- ;
    }
    argv++ ;
    argc-- ;
  }

/*
 *  Check and return ----------------------------------------------------
 */
  printf ("fBadd: %d files and %d offsets\n", *nfiles, i) ;
  if ( !outFile[0] || !(*nfiles) ) {
    fprintf (stderr, "fBadd - Usage:\n    %s ",  progName) ;
    fprintf (stderr, "-i inputFile1 [...] [-d offset1 [...]] -o outputFile\n") ;
    fprintf (stderr, "A list of filenames and offsets (required!) may be given in a parameter file through -f\n") ;
    return 1 ;
  }
  else
    return 0 ;
}
