/* RCS: $Id: fBfsum.c,v 3.1 1998/09/28 21:26:48 arots Exp arots $ */
/*-----------------------------------------------------------------------
 *
 *  fBfsum.c
 *
 *  Date: 21 March 1997
 *
 *  Arnold Rots, USRA
 *
 *  fBfsum allows summing phase bins in an fB file.  Errors are added in
 *  quadrature.
 *
 *
 *----------------------------------------------------------------------*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fitsio.h>
#include <longnam.h>
#define NCHAN   4096

char *fB = "fBfsum $Revision: 3.1 $" ;
int main (int argc, char *argv[])
{

  fitsfile *fits_infile;
  fitsfile *fits_outfile;
  char line[1024] ;
  char progName[256] ;
  char dataFile[1025] ;
  int phb[3] = {0, 0, 0} ;
  char outFile[1025] ;
  char keystrval[80] ;
  char comment[80] ;
  char *rid = comment ;
  int nfiles = 0 ;
  int error = 0 ;
  int htype ;
  long nrows, nch, i, j ;
  long nrows0, nch0, k, l ;
  double nulv = 0.0 ;
  long lnulv = 0 ;
  int any ;
  double dnphase, expos ;
  double dnphase0, expos0 ;
  double phase ;
  double inspec[NCHAN] ;
  double outspec[NCHAN] ;
  long numbers[NCHAN] ;
  double div ;

  int getopts (int, char **, char *, char *, int *, char *) ;

/*
 *  Decode command line options -----------------------------------------
 */
  printf ("# ") ;
  for (i=0; i<argc; i++)
    printf (" %s", argv[i]) ;
  printf ("\n") ;
  if ( getopts (argc, argv, progName, dataFile, phb, outFile) )
    exit (1) ;

/*
 *  Copy the input file -------------------------------------------------
 */
  if ( !strstr (outFile, ".") )
    strcat (outFile, ".pha") ;
  sprintf (line, "cp %s %s", dataFile, outFile) ;
  if ( system (line) ) {
    fprintf (stderr,"%s: Could not open input file %s or create output file %s\n",
	     progName, dataFile, outFile) ;
    exit (1) ;
  }

/*
 *  Fix up output file --------------------------------------------------
 */
  if ( fits_open_file (&fits_outfile, outFile, READWRITE, &error) ) {
    fprintf (stderr,"%s: Failed to re-open output file %s\n",
	     progName, outFile) ;
    exit (2) ;
  }
  fits_write_date (fits_outfile, &error) ;
  fits_write_chksum (fits_outfile, &error) ;

/*
 *  Check first extension -----------------------------------------------
 */
  fits_movabs_hdu (fits_outfile, 2, &htype, &error) ;
  fits_read_key (fits_outfile, TSTRING, "HDUCLAS5", keystrval, comment, &error) ;
  if ( strcmp (keystrval, "fB") ) {
    fprintf (stderr,"%s: Input file %s is of the wrong type\n",
	     progName, dataFile) ;
    fits_delete_file (fits_outfile, &error) ;
    exit (3) ;
  }
  fits_read_key_lng (fits_outfile,        "NAXIS2",    &nrows0, comment, &error) ;
  fits_read_key_dbl (fits_outfile,      "PHASEDEL",  &dnphase0, comment, &error) ;
  fits_read_key_lng (fits_outfile,      "DETCHANS",      &nch0, comment, &error) ;
  fits_delete_rows (fits_outfile, 1, nrows0, &error) ;
  if ( phb[0] <= 0 )
    phb[0] = 5 ;
  if ( ( phb[1] <= 0 ) || ( phb[1] > nrows0 ) )
    phb[1] = 1 ;
  if ( ( phb[2] < phb[1] ) || ( phb[2] > nrows0 ) )
    phb[2] = nrows0 ;
  if ( phb[0] > (phb[2] - phb[1] + 1) )
    phb[0] = phb[2] - phb[1] + 1 ;
  nrows0 = (phb[2] - phb[1] + 1) / phb[0] ;
  dnphase0 *= phb[0] ;
  div = (double) 1.0 / phb[0] ;
  fits_modify_key_lng (fits_outfile,     "NAXIS2",   nrows0,    "&", &error) ;
  fits_modify_key_dbl (fits_outfile,   "PHASEDEL", dnphase0, 3, "&", &error) ;

/*
 *  Open input file -----------------------------------------------------
 */
    if ( fits_open_file (&fits_infile, dataFile, READONLY, &error) ) {
      fprintf (stderr,"%s: Failed to open input file %s\n",
	       progName, dataFile) ;
      exit (2) ;
    }
    fits_movabs_hdu (fits_infile, 2, &htype, &error) ;

/*
 *  Output row loop -----------------------------------------------------
 */
  fits_read_col (fits_infile, TLONG, 3, 1, 1,   nch0, &lnulv, &numbers,
		   &any, &error) ;
  j = phb[1] ;
  for (k=1; k<=nrows0; k++) {

  
/*
 *  Enter new values -----------------------------------------------------
 */
    fits_write_col_lng (fits_outfile, 1, k, 1,    1, &k,      &error) ;
    fits_write_col_lng (fits_outfile, 3, k, 1, nch0, numbers, &error) ;

/*
 *  Phase ------------------------------------------------------
 */
    fits_read_col (fits_infile, TDOUBLE, 2, j, 1,   1, &nulv, &expos0,
		   &any, &error) ;
    for (l=1; l<phb[0]; l++) {
      fits_read_col ( fits_infile, TDOUBLE, 2, j+l, 1,   1, &nulv,  &expos,
		     &any, &error) ;
      expos0 += expos ;
    }
    expos0 *= div ;
    fits_write_col_dbl (fits_outfile, 2, k, 1,   1, &expos0, &error) ;
    sprintf (comment, "Phase %5.3g", expos0) ;
    fits_write_col_str (fits_outfile, 8, k, 1,   1,    &rid, &error) ;
/*
 *  Counts -----------------------------------------------------
 */
    fits_read_col (fits_infile, TDOUBLE, 4, j, 1, nch0, &nulv, outspec,
		   &any, &error) ;
    for (l=1; l<phb[0]; l++) {
      fits_read_col ( fits_infile, TDOUBLE, 4, j+l, 1, nch0, &nulv,  inspec,
		     &any, &error) ;
      for (i=0; i<nch0; i++)
	outspec[i] += inspec[i] ;
    }
    fits_write_col_dbl (fits_outfile, 4, k, 1, nch0, outspec, &error) ;
/*
 *  Errors -----------------------------------------------------
 */
    fits_read_col (fits_infile, TDOUBLE, 5, j, 1, nch0, &nulv, outspec,
		   &any, &error) ;
    for (l=1; l<phb[0]; l++) {
      fits_read_col ( fits_infile, TDOUBLE, 5, j+l, 1, nch0, &nulv,  inspec,
		     &any, &error) ;
      for (i=0; i<nch0; i++)
	outspec[i] = sqrt (outspec[i]*outspec[i] + inspec[i]*inspec[i]) ;
    }
    fits_write_col_dbl (fits_outfile, 5, k, 1, nch0, outspec, &error) ;
/*
 *  Baseline ---------------------------------------------------
 */
    fits_read_col (fits_infile, TDOUBLE, 6, j, 1, nch0, &nulv, outspec,
		   &any, &error) ;
    for (l=1; l<phb[0]; l++) {
      fits_read_col ( fits_infile, TDOUBLE, 6, j+l, 1, nch0, &nulv,  inspec,
		     &any, &error) ;
      for (i=0; i<nch0; i++)
	outspec[i] += inspec[i] ;
    }
    fits_write_col_dbl (fits_outfile, 6, k, 1, nch0, outspec, &error) ;
/*
 *  Exposure ---------------------------------------------------
 */
    fits_read_col (fits_infile, TDOUBLE, 7, j, 1,   1, &nulv, &expos0,
		   &any, &error) ;
    for (l=1; l<phb[0]; l++) {
      fits_read_col ( fits_infile, TDOUBLE, 7, j+l, 1,   1, &nulv,  &expos,
		     &any, &error) ;
      expos0 += expos ;
    }
    fits_write_col_dbl (fits_outfile, 7, k, 1,   1, &expos0, &error) ;

  j += phb[0] ;
  }  
/*
 *  Last fixes and close ------------------------------------------------
 */
  fits_write_date (fits_outfile, &error) ;
  fits_modify_key_str (fits_outfile, "CREATOR", fB, "&", &error) ;
  fits_write_chksum (fits_outfile, &error) ;

  fits_close_file (fits_outfile, &error) ;

  exit (error) ;
}

/*-----------------------------------------------------------------------
 *
 *  getopts decodes command line options for faseBin.
 *
 *----------------------------------------------------------------------*/

int getopts (int argc, char **argv, char *progName, char *dataFile,
	     int *phb, char *outFile)
{
  int i ;
/*
 *  Initialize ----------------------------------------------------------
 */
  dataFile[0] = 0 ;
  outFile[0] = 0 ;
  strcpy (progName, *argv) ;
  argv++ ;
  argc-- ;

/*
 *  Decode --------------------------------------------------------------
 */
  while ( argc > 0 ) {
    if ( !strcmp (*argv, "-i") ) {
      strcpy (dataFile, *(++argv)) ;
      argc-- ;
    }
    else if ( !strcmp (*argv, "-o") ) {
      strcpy (outFile, *(++argv)) ;
      argc-- ;
    }
    else if ( !strcmp (*argv, "-phbins") ) {
      i = 0 ;
      argc-- ;
      while ( argc && ( **(++argv) != '-' ) ) {
	phb[i] = atoi (*argv) ;
	argc-- ;
	i++ ;
      }
      argv-- ;
      argc++ ;
    }
    argv++ ;
    argc-- ;
  }

/*
 *  Check and return ----------------------------------------------------
 */
  if ( !(*dataFile) ) {
    fprintf (stderr, "fBfsum - Usage:\n    %s ",  progName) ;
    fprintf (stderr, "-i inputFile -o outputFile [-phbins step [start [stop]]]\n") ;
    return 1 ;
  }
  else
    return 0 ;
}
