/* RCS: $Id: faseBin.c,v 3.6 2012/05/02 18:53:56 arots Exp arots $ */
/*-----------------------------------------------------------------------
 *
 *  faseBin.c
 *
 *  Date: 19 March 1997
 *
 *  Arnold Rots, USRA
 *
 *  faseBin is a program applying barycenter corrections to XTE event
 *  observations, calculating the phase, binning in phase/pha space,
 *  and writing the result as a pha (fB) file.
 *
 *
 *----------------------------------------------------------------------*/

#include "bary.h"
#define NPHASE  1000
#define NPHASE0 100

char *fB = "faseBin $Revision: 3.6 $" ;
int main (int argc, char *argv[])
{

  fitsfile *fits_file;
  char *ttype[8] = {"SPEC_NUM", "PHASE", "CHANNEL", "COUNTS", "STAT_ERR",
		      "BASELINE", "EXPOSURE", "ROWID"} ;
  char *tunit[8] = {"", "", "", "count", "count", "count", "s", ""} ;
  char *tform[8] ;
  char telescope[80] ;
  char instrument[80] ;
  char datareffrm[80] ;
  char plephem[80] ;
  double dir[3] ;
  char progName[1024] ;
  char orbitFile[1024] ;
  char *dataFile[1024] ;
  char gtiFile[1024] ;
  char respFile[1024] ;
  char outFile[1024] ;
  int nfiles = 0 ;
  char source[25] ;
  char line[256] ;
  char cpix[4096] ;
  char datobs[80] = "" ;
  int gainapp = 0 ;
  char *subline ;
  double *start ;
  double *stop ;
  double tstart = 1.0e20 ;
  double tstop = 0.0 ;
  double tstt, tstp ;
  double *exposure ;
  double *exposur2 ;
  double totexposure ;
  int nch = 0   ;
  int row = 0 ;
  int debug = 0 ;
  int nTR = 1 ;
  long nrows = 0 ;
  int anynul ;
  double nulval = 0.0 ;
  char filetype[32] ;
  int htype ;
  int accum = 0 ;
  int nevents = 0 ;
  double ra = -100.0 ;
  double dec = -100.0 ;
  double t1, t2 ;
  int binary = 0 ;
  double **image ;
  double *pimage ;
  long **img ;
  int l1only = 1 ;
  int nobcorr = 0 ;

  long numbers[256] ;
  char rowid[24] ;
  char *rid = rowid ;
  int channel ;
  int nphase = NPHASE0 ;
  double dnphase ;
  double t ;
  int error = 0 ;
  double tZero = 0.0 ;
  long i, n, j, k ;
  int gotparms = 0 ;
  double radioph, roffset = 0.0 ;
  double f0 ;
  double x ;
  float v ;
  PsrTimPar *inptp=NULL ;
  PsrBinPar *inpbp=NULL ;
  int getopts (int, char **, char *, char *, char **, int *, char *,
	       double *, char *, char *, char *, int *, int *, int *, int *) ;

/*
 *  Set defaults --------------------------------------------------------
 */
  strcpy (outFile, "faseBin.pha") ;
  strcpy (respFile, "none") ;
  strcpy (gtiFile, "none") ;
  strcpy (filetype, "none") ;
  strcpy (cpix, "none") ;

/*
 *  Decode command line options -----------------------------------------
 */
  printf ("# ") ;
  for (i=0; i<argc; i++)
    printf (" %s", argv[i]) ;
  printf ("\n") ;
  if ( getopts (argc, argv, progName, orbitFile, dataFile, &nfiles, source,
		&roffset, gtiFile, respFile, outFile,
		&nphase, &binary, &l1only, &debug) )
    exit (1) ;

  if ( ( nphase < 1 ) || ( nphase > NPHASE ) )
    nphase = NPHASE0 ;
  if ( !strstr (outFile, ".") )
    strcat (outFile, ".pha") ;
  if ( debug )
    fprintf (stderr, "%s: %s\n%s: cfitsio version %f\n",
	     progName, fB, progName, fits_get_version (&v) ) ;

/*
 *  Read GTI table ------------------------------------------------------
 */
  nrows = 0 ;
  if ( strcmp (gtiFile, "none") ) {
    if ( fits_open_file (&fits_file, gtiFile, READONLY, &error) ) {
      fprintf(stderr, "%s: cannot open file %s\n", progName, gtiFile) ;
      exit (2) ;
    }
    while ( strcmp(filetype, "GTI") && !error ) {
      fits_movrel_hdu (fits_file, 1, &htype, &error) ;
      fits_read_key   (fits_file, TSTRING, "HDUCLAS1",  filetype, line, &error) ;
      if ( debug )
	fprintf (stderr, "%s: GTI file %s HDUCLAS1 %s\n",
		 progName, gtiFile, filetype) ;
    }
    fits_read_key   (fits_file, TINT,    "NAXIS2",   &nrows,   line, &error) ;
  }
  if ( nrows && !error ) {
    fits_read_key   (fits_file, TDOUBLE, "TIMEZERO", &tZero,   line, &error) ;
    if ( error ) {
      tZero = 0.0 ;
      error = 0 ;
    }
    if ( debug )
      fprintf (stderr, "%s: GTI file %s nrows %d tZero %f\n",
		 progName, gtiFile, nrows, tZero) ;
    start = (double*) malloc (nrows * sizeof(double)) ;
    stop = (double*) malloc (nrows * sizeof(double)) ;
    fits_read_col (fits_file, TDOUBLE, 1, 1, 1, nrows,
		   &nulval, start, &anynul, &error) ;
    fits_read_col (fits_file, TDOUBLE, 2, 1, 1, nrows,
		   &nulval, stop, &anynul, &error) ;
    for (i=0; i<nrows; i++) {
      start[i] += tZero ;
      stop[i] += tZero ;
    }
    nTR = nrows ;
  }
  else {
    start = (double*) malloc (sizeof(double)) ;
    stop = (double*) malloc (sizeof(double)) ;
    *start = -1.0e20 ;
    *stop = 1.0e20 ;
    nTR = 1 ;
  }
  if ( strcmp (gtiFile, "none") )
    fits_close_file (fits_file, &error) ;
  error = 0 ;
  if ( debug )
    fprintf (stderr, "%s: finished GTI handling (%s)\n",
	     progName, gtiFile) ;

/*
 *  Open output file ----------------------------------------------------
 */
  if ( !(fits_open_file (&fits_file, outFile, READONLY, &error) ) )
    fits_delete_file ( fits_file, &error) ;
  error = 0 ;
  if ( fits_create_file (&fits_file, outFile, &error) ) {
    fprintf(stderr, "%s: cannot create file %s\n", progName, outFile) ;
    exit (3) ;
  }
  if ( debug )
    fprintf (stderr, "%s: created output file (%s)\n",
	     progName, outFile) ;
  n = 0 ;
  fits_create_img (fits_file, 8, 0, &n, &error) ;
  fits_set_hdustruc (fits_file, &error) ;
  fits_write_chksum (fits_file, &error) ;

/*
 *  Do it ---------------------------------------------------------------
 */
  for (i=0; i<nfiles; i++) {
    tZero = 0.0 ;
    img = phaseHist (orbitFile, dataFile[i], telescope, instrument,
		     datareffrm, plephem,
		     source, &ra, &dec, roffset, &tZero,
		     binary, inptp, inpbp, l1only, nTR, start, stop,
		     &tstt, &tstp, &exposure, accum, &gainapp, datobs,
		     &nphase, &nch, cpix, &f0, nobcorr, &nevents, debug) ;
    if ( debug )
      fprintf (stderr, "%s: execution %d of phaseHist\n",
	       progName, i) ;
    accum = 1 ;
    if ( tstart > tstt )
      tstart = tstt ;
    if ( tstop < tstp )
      tstop = tstp ;
  }
  dnphase = (double) 1.0 / nphase ;

/*
 *  Create the bintable -------------------------------------------------
 */
  nrows = 2*nphase+1 ;
  if ( debug )
    fprintf (stderr, "%s: trying to allocate tforms, %d rows\n", progName, nrows) ;
  *tform = (char *) malloc (160*sizeof(char)) ;
  if ( debug )
    fprintf (stderr, "%s: allocated tforms\n", progName) ;
  for (i=1; i<8; i++)
    tform[i] = tform[0] + 20*i ;
  if ( debug )
    fprintf (stderr, "%s: start creating TFORMs, %d chns\n", progName, nch) ;
  strcpy (tform[0], "J") ;
  strcpy (tform[1], "D") ;
  sprintf (tform[2], "%dJ", nch) ;
  sprintf (tform[3], "%dD", nch) ;
  sprintf (tform[4], "%dD", nch) ;
  sprintf (tform[5], "%dD", nch) ;
  strcpy (tform[6], "D") ;
  strcpy (tform[7], "20A") ;
  if ( debug )
    fprintf (stderr, "%s: start creating bintable, %d rows\n", progName, nrows) ;
  fits_create_tbl (fits_file, BINARY_TBL, nrows, 8,
		   ttype, tform, tunit, "SPECTRUM", &error) ;
  if ( debug )
    fprintf (stderr, "%s: created bintable\n", progName) ;
  fits_write_comment (fits_file, "===== HDU configuration information", &error) ;
  fits_write_date (fits_file, &error) ;
  strcpy (line, fitsdate()) ;
  fits_update_key (fits_file, TSTRING,     "DATE",       line, "", &error) ;
  fits_write_key (fits_file, TSTRING,  "CREATOR",         fB, "", &error) ;
  fits_write_key (fits_file, TSTRING, "HDUCLASS",     "OGIP", "", &error) ;
  fits_write_key (fits_file, TSTRING, "HDUCLAS1", "SPECTRUM", "", &error) ;
  fits_write_key (fits_file, TSTRING,  "HDUVERS",    "1.1.0", "", &error) ;
  fits_write_key (fits_file, TSTRING, "HDUCLAS2", "ACCEPTED", "", &error) ;
  fits_write_key (fits_file, TSTRING, "HDUCLAS3",  "TYPE:II", "", &error) ;
  fits_write_key (fits_file, TSTRING, "HDUCLAS4",    "COUNT", "", &error) ;
  fits_write_key (fits_file, TSTRING, "HDUCLAS5",       "fB", "", &error) ;
  fits_write_key_longwarn (fits_file, &error) ;
  fits_write_comment (fits_file, "===== Observation - Object, coordinate, time information", &error) ;
  fits_write_key (fits_file, TSTRING,   "OBJECT",     source, "", &error) ;
  fits_write_key (fits_file, TSTRING, "TELESCOP",  telescope, "", &error) ;
  fits_write_key (fits_file, TSTRING, "INSTRUME", instrument, "", &error) ;
  fits_write_key (fits_file, TSTRING,   "FILTER",     "none", "", &error) ;
  fits_write_key_fixdbl (fits_file,     "RA-OBJ",      ra, 9, " \\  Used", &error) ;
  fits_write_key_fixdbl (fits_file,    "DEC-OBJ",     dec, 9, "  | for", &error) ;
  fits_write_key_fixdbl (fits_file,    "EQUINOX",  2000.0, 1, "  | barycentric", &error) ;
  fits_write_key (fits_file, TSTRING, "RADECSYS", datareffrm, " /  corrections", &error) ;
  fits_write_key (fits_file, TSTRING,  "PLEPHEM",    plephem, "Solar system ephem used for barycenter", &error) ;
  fits_write_key (fits_file, TSTRING,  "TIMESYS",      "TDB", " \\ Barycentric", &error) ;
  fits_write_key (fits_file, TSTRING,  "TIMEREF","SOLARSYSTEM", " / corrections applied", &error) ;
  fits_write_key (fits_file, TSTRING, "TIMEUNIT",        "d", "Unit of TSTART/TSTOP is days", &error) ;
  fits_write_key_fixdbl (fits_file,     "MJDREF",     0.0, 1, "Start and stop time are directly in MJD", &error) ;
  fits_write_key_fixdbl (fits_file,     "TSTART", tstart, 12, "Start of data in MJD", &error) ;
  fits_write_key_fixdbl (fits_file,      "TSTOP",  tstop, 12, "End of data in MJD", &error) ;
  fits_write_comment (fits_file, "Modified Julian Day (MJD) = JD - 2400000.5", &error) ;
  fits_write_key (fits_file, TSTRING, "DATE-OBS",     datobs, "", &error) ;
  fits_write_key_fixdbl (fits_file,   "PHASEDEL", dnphase, 5, "Phase bin size", &error) ;
  fits_write_comment (fits_file, "===== Spectral and calibration information", &error) ;
  fits_write_key_lng (fits_file,      "DETCHANS",        nch, "", &error) ;
  fits_write_key_lng (fits_file,        "TLMIN3",          0, "", &error) ;
  fits_write_key_lng (fits_file,        "TLMAX3",      nch-1, "", &error) ;
  fits_write_key_longstr (fits_file,     "CPIX4",       cpix, "", &error) ;
  fits_write_comment (fits_file, "CPIX4 contains the translation of the spectral binning", &error) ;
  fits_write_key (fits_file, TSTRING, "BACKFILE",     "none", "", &error) ;
  fits_write_key (fits_file, TSTRING, "CORRFILE",     "none", "", &error) ;
  fits_write_key (fits_file, TSTRING, "RESPFILE",   respFile, "", &error) ;
  fits_write_key (fits_file, TSTRING, "ANCRFILE",     "none", "", &error) ;
  fits_write_key (fits_file, TSTRING, "CHANTYPE",      "PHA", "", &error) ;
  fits_write_key_log (fits_file,      "POISSERR",          0, "", &error) ;
  fits_write_key_fixdbl (fits_file,   "AREASCAL",     1.0, 1, "", &error) ;
  fits_write_key_fixdbl (fits_file,   "BACKSCAL",     1.0, 1, "", &error) ;
  fits_write_key_fixdbl (fits_file,   "CORRSCAL",     0.0, 1, "", &error) ;
  fits_write_key_log (fits_file,       "GAINAPP",    gainapp, "", &error) ;
  fits_modify_comment (fits_file, "TTYPE1", "Sequence number of spectrum", &error) ;
  fits_modify_comment (fits_file, "TTYPE2", "Phase of spectrum", &error) ;
  fits_modify_comment (fits_file, "TTYPE3", "Channel list of spectrum", &error) ;
  fits_modify_comment (fits_file, "TTYPE4", "Spectrum", &error) ;
  fits_modify_comment (fits_file, "TTYPE5", "Statistical errors of spectral bins", &error) ;
  fits_modify_comment (fits_file, "TTYPE6", "Baseline (background) counts for spectral bins", &error) ;
  fits_modify_comment (fits_file, "TTYPE7", "Total exposure for spectrum", &error) ;
  fits_modify_comment (fits_file, "TTYPE8", "Row label", &error) ;
  if ( debug )
    fprintf (stderr, "%s: completed header\n", progName) ;

/*
 *  Fill the "data" column ----------------------------------------------
 */
  image = (double **) malloc (nrows * sizeof(double*)) ;
  *image = (double *) malloc (nrows * nch * sizeof(double)) ;
  for (i=1; i<nrows; i++)
    image[i] = *image + i * nch ;
  for (j=0; j<nch; j++) {
    for (i=0; i<nphase; i++)
      image[i][j] = image[i+nphase][j] = (double) img[j][i] ;
    image[2*nphase][j] = (double) img[j][0] ;
  }
  exposur2 = (double *) malloc (nrows * sizeof(double)) ;
  for (i=0; i<nphase; i++)
    exposur2[i] = exposur2[i+nphase] = exposure[i] ;
  exposur2[2*nphase] = exposur2[0] ;
  if ( debug )
    fprintf (stderr, "%s: filled data\n", progName) ;

/*
 *  Write to output file ------------------------------------------------
 */
  fits_write_col_dbl (fits_file, 4, 1, 1, nrows*nch,   *image, &error) ;
  if ( debug )
    fprintf (stderr, "%s: wrote counts\n", progName) ;
  for (i=0, pimage=*image; i<nrows*nch; i++, pimage++)
    *pimage = (*pimage > 0.0 ) ? sqrt(*pimage) : 0.0 ;
  fits_write_col_dbl (fits_file, 5, 1, 1, nrows*nch,   *image, &error) ;
  if ( debug )
    fprintf (stderr, "%s: wrote errors\n", progName) ;
  for (i=0, pimage=*image; i<nrows*nch; i++, pimage++)
    *pimage = 0.0 ;
  fits_write_col_dbl (fits_file, 6, 1, 1, nrows*nch,   *image, &error) ;
  fits_write_col_dbl (fits_file, 7, 1, 1,     nrows, exposur2, &error) ;
  if ( debug )
    fprintf (stderr, "%s: wrote zero and exp\n", progName) ;

  for (i=0; i<256; i++)
    numbers[i] = i ;
  for (i=0; i<nrows; i++) {
    x = (double) i*dnphase ;
    sprintf (rowid, "Phase %5.3g", x) ;
    fits_write_col_lng (fits_file, 1, i+1, 1,   1,      &i, &error) ;
    fits_write_col_dbl (fits_file, 2, i+1, 1,   1,      &x, &error) ;
    fits_write_col_lng (fits_file, 3, i+1, 1, nch, numbers, &error) ;
    fits_write_col_str (fits_file, 8, i+1, 1,   1,    &rid, &error) ;
  }
  if ( debug )
    fprintf (stderr, "%s: wrote scalars\n", progName) ;

/*
 *  Close up and exit ---------------------------------------------------
 */
  fits_write_chksum (fits_file, &error) ;
  fits_close_file (fits_file, &error) ;
  totexposure = 0.0 ;
  for (i=0; i<nphase; i++)
    totexposure += exposure[i] ;
  printf ("# Input file(s):\n") ;
  for (i=0; i<nfiles; i++)
    printf ("#   %s\n", dataFile[i]) ;
  printf ("# Accumulated %d events during an exposure time of %f s\n",
	  nevents, totexposure) ;
  printf ("# Total clock correction: %f s\n", tZero) ;

  free (start) ;
  free (stop) ;
  free (exposur2) ;
  free (*tform) ;
  free (*image) ;
  free (image) ;
  for (i=0; i<nfiles; i++)
    free (dataFile[i]) ;

  exit (0) ;
}

/*-----------------------------------------------------------------------
 *
 *  getopts decodes command line options for faseBin.
 *
 *----------------------------------------------------------------------*/

int getopts (int argc, char **argv, char *progName, char *orbitFile,
	     char **dataFile, int *nfiles, char *source, double *roffset,
	     char *gtiFile, char *respFile, char *outFile,
	     int *nphase, int *binary, int *l1only, int *debug)
{
/*
 *  Initialize ----------------------------------------------------------
 */
  orbitFile[0] = 0 ;
  source[0] = 0 ;
  strcpy (progName, *argv) ;
  argv++ ;
  argc-- ;

/*
 *  Decode --------------------------------------------------------------
 */
  while ( argc > 0 ) {
    if ( !strcmp (*argv, "-debug") ) {
      if ( ( argc == 1 ) || ( **(argv+1) == '-' ) )
	*debug = 1 ;
      else {
	*debug = atoi (*(++argv)) ;
	argc-- ;
      }
    }
    else if ( !strcmp (*argv, "-binary") )
      *binary = 1 ;
    else if ( !strncmp (*argv, "-roff", 5) ) {
      *roffset = atof (*(++argv)) ;
      argc-- ;
    }
    else if ( !strcmp (*argv, "-i") ) {
      strcpy (orbitFile, *(++argv)) ;
      argc-- ;
    }
    else if ( !strcmp (*argv, "-o") ) {
      strcpy (outFile, *(++argv)) ;
      argc-- ;
    }
    else if ( !strcmp (*argv, "-f") ) {
      argc-- ;
      while ( ( **(++argv) != '-' ) && argc ) {
	dataFile[*nfiles] = (char*) malloc(1025 * sizeof(char)) ;
	strcpy (dataFile[*nfiles], *argv) ;
	argc-- ;
	(*nfiles)++ ;
      }
      argv-- ;
      argc++ ;
    }
    else if ( !strcmp (*argv, "-resp") ) {
      strcpy (respFile, *(++argv)) ;
      argc-- ;
    }
    else if ( !strcmp (*argv, "-gti") ) {
      strcpy (gtiFile, *(++argv)) ;
      argc-- ;
    }
    else if ( !strcmp (*argv, "-src") ) {
      strcpy (source, *(++argv)) ;
      argc-- ;
    }
    else if ( !strcmp (*argv, "-nph") ) {
      *nphase = atoi (*(++argv)) ;
      argc-- ;
    }
    else if ( !strcmp (*argv, "-l1") ) {
      *l1only = atoi (*(++argv)) ;
      argc-- ;
    }
    argv++ ;
    argc-- ;
  }

/*
 *  Check and return ----------------------------------------------------
 */
  if ( !orbitFile[0] || !source[0] || !*nfiles ) {
    fprintf (stderr, "FaseBin - Usage:\n    %s ",  progName) ;
    fprintf (stderr, "-i orbitFile -src sourceName -f dataFile [...] [-o outFile]\n") ;
    fprintf (stderr, "                               [-gti gtiFile] [-resp respFile]\n") ;
    fprintf (stderr, "                               [-roff radioOffset] [-binary]\n") ;
    fprintf (stderr, "                               [-nph nphase] [-l1 l1only]\n") ;

    fprintf (stderr, "Output goes to outFile (default: faseBin.pha).\n") ;
    fprintf (stderr, "nphase is the number of bins to use (<= %d)\n", NPHASE) ;
    fprintf (stderr, "radioOffset is the offset (in seconds!) as published in the radio database.\n") ;
    fprintf (stderr, "Option -binary will force searching the binary orbit table\n") ;
    fprintf (stderr, "l1only restricts analysis to first layer (GoodXenon only)\n") ;
    return 1 ;
  }
  else
    return 0 ;
}
